@@ -1,13 +1,12 @@
-///////////////////////////////////////////////////////////////////////////////
-// Name:        minimal.cpp
-// Purpose:     Test program for the wxSQLite3 class
-//              This example is based on the CppSQLite example.
-// Author:      Ulrich Telle
-// Modified by:
-// Created:     2005-07-14
-// Copyright:   (c) Ulrich Telle
-// Licence:     wxWindows licence
-///////////////////////////////////////////////////////////////////////////////
+/*
+** Name:        minimal.cpp
+** Purpose:     Test program for the wxSQLite3 class
+**              This example is based on the CppSQLite example.
+** Author:      Ulrich Telle
+** Created:     2005-07-14
+** Copyright:   (c) 2005-2024 Ulrich Telle
+** License:     LGPL-3.0+ WITH WxWindows-exception-3.1
+*/
 
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
@@ -20,6 +19,13 @@
 #include "wx/wx.h"
 #endif
 
+// Enable to activate leak detection with Visual Leak Detector
+#if 0
+#include <vld.h>
+#endif
+
+#include <wx/cmdline.h>
+
 #include "wx/wxsqlite3.h"
 #include <ctime>
 #include <iostream>
@@ -28,27 +34,27 @@ using namespace std;
 
 // Test of RAII transaction class
 
-wxSQLite3Database* initDB(void)
+static wxSQLite3Database* initDB(void)
 {
-	wxString testDBName = wxGetCwd() + wxT("/test2.db");
+	wxString testDBName = wxGetCwd() + wxS("/test2.db");
 	if (wxFileExists(testDBName))
 	{
 		wxRemoveFile(testDBName);
 	}
 	wxSQLite3Database* db = new wxSQLite3Database();
 	db->Open(testDBName);
-	db->ExecuteUpdate(wxT("CREATE TABLE test (col1 INTEGER)"));
+	db->ExecuteUpdate(wxS("CREATE TABLE test (col1 INTEGER)"));
 	return db;
 }
 
-void clearDB(wxSQLite3Database* db)
+static void clearDB(wxSQLite3Database* db)
 {
 	assert(db != NULL);
 	db->Close();
 	delete db;
 }
 
-void testTransaction()
+static void testTransaction()
 {
   bool exceptionCaught = false;
   wxSQLite3Database* db = initDB();
@@ -57,7 +63,7 @@ void testTransaction()
 		wxSQLite3Transaction t(db);
     cout << "AutoCommit? " << !db->GetAutoCommit() << endl;
 		cout << "Transaction active? " << t.IsActive() << endl;
-		db->ExecuteUpdate(wxT("INSERT INTO test (col1) VALUES (2)"));
+		db->ExecuteUpdate(wxS("INSERT INTO test (col1) VALUES (2)"));
 		t.Commit();
     cout << "AutoCommit? " << db->GetAutoCommit() << endl;
 		cout << "Transaction not active? " << !t.IsActive() << endl;
@@ -67,7 +73,7 @@ void testTransaction()
 		cout << "Exception should not happen here" << endl;
 	}
 	// Check whether value exists in table
-	wxSQLite3ResultSet set = db->ExecuteQuery(wxT("SELECT * FROM test"));
+	wxSQLite3ResultSet set = db->ExecuteQuery(wxS("SELECT * FROM test"));
 	
 	int count = 0;
 	while (set.NextRow())
@@ -82,7 +88,8 @@ void testTransaction()
 	try
 	{
 		wxSQLite3Transaction t(db);
-		db->ExecuteUpdate(wxT("INSERT INTO test (col1) VALUES (3)"));
+		db->ExecuteUpdate(wxS("INSERT INTO test (col1) VALUES (3)"));
+
 		throw "Abort commit";
 //  ...
 //  t.Commit();
@@ -95,7 +102,7 @@ void testTransaction()
   {
 		// check whether the value 3 exists in table
     // (it shouldn't since the transaction was aborted)
-		set = db->ExecuteQuery(wxT("SELECT * FROM test"));
+		set = db->ExecuteQuery(wxS("SELECT * FROM test"));
 	
 		int count = 0;
 		while (set.NextRow())
@@ -116,7 +123,6 @@ void testTransaction()
 	clearDB(db);
 }
 
-
 // User defined aggregate function
 class MyAggregateFunction : public wxSQLite3AggregateFunction
 {
@@ -129,13 +135,13 @@ public:
 
     // Allocate a wxString instance in the first aggregate step
     if (*acc == NULL) {
-      *acc = new wxString(wxT(""));
+      *acc = new wxString(wxS(""));
     }
 
     // Concatenate all arguments
     for (int i = 0; i < ctx.GetArgCount(); i++) {
        (*acc)->Append(ctx.GetString(i));
-       (*acc)->Append(wxT(" "));
+       (*acc)->Append(wxS(" "));
     }
   }
 
@@ -162,14 +168,16 @@ class MyAuthorizer : public wxSQLite3Authorizer
 public:
   virtual wxAuthorizationResult Authorize(wxAuthorizationCode type, 
                                           const wxString& arg1, const wxString& arg2, 
-                                          const wxString& arg3, const wxString& arg4)
+                                          const wxString& arg3, const wxString& arg4,
+                                          const wxString& arg5)
   {
     cout << "AUTH: " 
          << (const char*) AuthorizationCodeToString(type).mb_str(wxConvUTF8) << ","
          << (const char*) arg1.mb_str(wxConvUTF8) << ","
          << (const char*) arg2.mb_str(wxConvUTF8) << ","
          << (const char*) arg3.mb_str(wxConvUTF8) << ","
-         << (const char*) arg4.mb_str(wxConvUTF8) << endl;
+         << (const char*) arg4.mb_str(wxConvUTF8) << ","
+         << (const char*) arg5.mb_str(wxConvUTF8) << endl;
     return wxSQLite3Authorizer::SQLITE_OK;
   }
 };
@@ -252,23 +260,71 @@ public:
   bool OnInit();
   int OnRun();
   int OnExit();
+  void TestSQLCipher(wxSQLite3Cipher& cipher, const wxString& dbFileName, const wxString& dbKey);
+  void TestPersonsDb(wxSQLite3Cipher& cipher, const wxString& dbFileName, const wxString& dbKey);
+  void TestCiphers();
 private:
+  wxString m_workDirectory;
+  bool     m_testMode;
+  int      m_rc;
 };
 
+static const wxCmdLineEntryDesc cmdLineDesc[] =
+{
+#if wxCHECK_VERSION(2,9,0)
+  { wxCMD_LINE_OPTION, "s", "sampledir", "wxSQLite3 samples directory", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
+  { wxCMD_LINE_SWITCH, "t", "testmode",  "Non-interactive testmode",    wxCMD_LINE_VAL_NONE,   wxCMD_LINE_PARAM_OPTIONAL },
+#else
+  { wxCMD_LINE_OPTION, wxS("s"), wxS("sampledir"), wxS("wxSQLite3 samples directory"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL },
+  { wxCMD_LINE_SWITCH, wxS("t"), wxS("testmode"),  wxS("Non-interactive testmode"),    wxCMD_LINE_VAL_NONE,   wxCMD_LINE_PARAM_OPTIONAL },
+#endif
+  { wxCMD_LINE_NONE }
+};
+
+
 bool Minimal::OnInit()
 {
-  return true;
+  m_rc = 0;
+  m_testMode = false;
+  // Gets the parameters from cmd line
+  wxCmdLineParser parser(cmdLineDesc, argc, argv);
+  wxString logo = wxS("wxSQLite3 Minimal Sample\n");
+  parser.SetLogo(logo);
+  bool ok = parser.Parse() == 0;
+  if (ok)
+  {
+    parser.Found(wxS("sampledir"), &m_workDirectory);
+    m_testMode = parser.Found(wxS("testmode"));
+    m_rc = 0;
+  }
+  else
+  {
+    m_rc = -1;
+  }
+
+  if (ok)
+  {
+    // Set the working directory
+    if (!m_workDirectory.IsEmpty())
+    {
+      wxSetWorkingDirectory(m_workDirectory);
+    }
+    m_workDirectory = wxGetCwd();
+  }
+
+  return ok;
 }
 
 int Minimal::OnExit()
 {
-  return 0;
+  return m_rc;
 }
 
 int Minimal::OnRun()
 {
-  const wxString dbFile = wxGetCwd() + wxT("/test.db");
-  const wxString dbBackup = wxGetCwd() + wxT("/test-backup.db");
+  m_rc = 0;
+  const wxString dbFile = wxGetCwd() + wxS("/test.db");
+  const wxString dbBackup = wxGetCwd() + wxS("/test-backup.db");
 
   const char* rawUTF8 = "\xe2\x8c\x88\x30\x2e\x35\x6e\xe2\x8c\x89\x2b\x6e\x2b\x32\x36";
   wxString tt = wxString::FromUTF8(rawUTF8);
@@ -303,18 +359,29 @@ int Minimal::OnRun()
     }
 
     // Remove existing sample database files
-    wxRemoveFile(dbFile);
-    wxRemoveFile(dbBackup);
+    if (wxFileExists(dbFile))
+    {
+      wxRemoveFile(dbFile);
+    }
+    if (wxFileExists(dbBackup))
+    {
+      wxRemoveFile(dbBackup);
+    }
 
     if (wxSQLite3Database::HasEncryptionSupport())
     {
-      db.Open(dbFile, wxString(wxT("password")));
+      cout << "Cipher (default) : " << (const char*) wxSQLite3Cipher::GetCipherName(wxSQLite3Cipher::GetGlobalCipherDefault()).mb_str(wxConvUTF8) << endl;
+      db.Open(dbFile, wxString(wxS("password")));
     }
     else
     {
       db.Open(dbFile);
     }
 
+    int cfgResult;
+    db.Configure(WXSQLITE_DBCONFIG_DEFENSIVE, 1, cfgResult);
+    cout << "SQLite3 Configuration: Defensive mode is " << ((cfgResult != 0) ? "enabled" : "disabled") << endl;
+
     cout << "SQLite3 Journal Mode: " << (const char*) wxSQLite3Database::ConvertJournalMode(db.GetJournalMode()).mb_str(wxConvUTF8) << endl;
     // Check status of support for foreign key constraints
     bool foreignKeysEnabled = db.EnableForeignKeySupport(true);
@@ -325,34 +392,56 @@ int Minimal::OnRun()
       cout << "not enforced.";
     cout << endl;
 
-    cout << endl << "emp table exists=" << (db.TableExists(wxT("EmP")) ? "TRUE":"FALSE") << endl;
-    cout << endl << "Creating emp table" << endl;
-    db.ExecuteUpdate(wxT("create table emp(empno int, empname char(20), salary double);"));
-    cout << endl << "emp table exists=" << (db.TableExists(wxT("emp")) ? "TRUE":"FALSE") << endl;
+    cout << endl << "Table 'emp' exists=" << (db.TableExists(wxS("EmP")) ? "TRUE":"FALSE") << endl;
+    cout << "Creating table 'emp'" << endl;
+    db.ExecuteUpdate(wxS("create table emp(empno int, empname char(20), salary double);"));
+    cout << "Table 'emp' exists=" << (db.TableExists(wxS("emp")) ? "TRUE":"FALSE") << endl;
     
     // Attach the current database under different name and
     // check table existance in any open database.
     // The table emp will be found in 'main' and in 'dbattached'
-    db.ExecuteUpdate(wxString(wxT("attach database '")) + dbFile + wxString(wxT("' as dbattached")));
+    db.AttachDatabase(dbFile, wxS("dbattached"));
+    cout << endl << "Filename of database 'dbattached': " << (const char*) db.GetDatabaseFilename(wxS("dbattached")).mb_str(wxConvUTF8) << endl;
+
     wxArrayString databaseList;
-    db.TableExists(wxT("emp"), databaseList);
+    db.TableExists(wxS("emp"), databaseList);
+    cout << "Table 'emp' exists in the following databases:" << endl;
     size_t j;
-    cout << endl << "emp table exists in the following databases:" << endl;
-    for (j = 0; j < databaseList.GetCount(); j++)
+    for (j = 0; j < databaseList.GetCount(); ++j)
     {
-      cout << j << ": " << (const char*) databaseList.Item(j).mb_str(wxConvUTF8) << endl;
+      cout << j << ": " << (const char*)databaseList.Item(j).mb_str(wxConvUTF8) << endl;
     }
 
-    cout << "Filename of attached database: " << (const char*) db.GetDatabaseFilename(wxT("dbattached")).mb_str(wxConvUTF8) << endl;
+    if (wxSQLite3Database::HasEncryptionSupport())
+    {
+      wxSQLite3CipherSQLCipher sqlCipher3;
+      sqlCipher3.InitializeVersionDefault(3);
+      db.AttachDatabase(wxS("sqlcipher-3.0-testkey.db"), wxS("dbsqlcipher"), sqlCipher3, wxS("testkey"));
+      cout << endl << "Filename of database 'dbsqlcipher': " << (const char*)db.GetDatabaseFilename(wxS("dbsqlcipher")).mb_str(wxConvUTF8) << endl;
 
-    db.ExecuteUpdate(wxT("detach database dbattached"));
+      databaseList.Empty();
+      db.TableExists(wxS("t1"), databaseList);
+      cout << "Table 't1' exists in the following databases:" << endl;
+      size_t k;
+      for (k = 0; k < databaseList.GetCount(); ++k)
+      {
+        cout << k << ": " << (const char*) databaseList.Item(k).mb_str(wxConvUTF8) << endl;
+      }
+    }
+
+    if (wxSQLite3Database::HasEncryptionSupport())
+    {
+      db.DetachDatabase(wxS("dbsqlcipher"));
+    }
+
+    db.DetachDatabase(wxS("dbattached"));
 
     // Execute some DML, and print number of rows affected by each one
 
     db.SetUpdateHook(&myCallback);
 
     cout << endl << "DML tests" << endl;
-    wxString insertCmd(wxT("insert into emp values (7, 'Franz Beckenbauer', 2000.10);"));
+    wxString insertCmd(wxS("insert into emp values (7, 'Franz Beckenbauer', 2000.10);"));
     int nRows = db.ExecuteUpdate(insertCmd);
     cout << nRows << " rows inserted" << endl;
 
@@ -399,8 +488,8 @@ int Minimal::OnRun()
     if (db.HasNamedCollectionSupport())
     {
       cout << endl << "Named collection test" << endl;
-      wxSQLite3IntegerCollection ic = db.CreateIntegerCollection(wxT("ic"));
-      wxSQLite3StringCollection sc = db.CreateStringCollection(wxT("sc"));
+      wxSQLite3IntegerCollection ic = db.CreateIntegerCollection(wxS("ic"));
+      wxSQLite3StringCollection sc = db.CreateStringCollection(wxS("sc"));
       int icList[] = { 26, 39, 47, 64 };
       ic.Bind(4, icList);
 
@@ -411,8 +500,8 @@ int Minimal::OnRun()
       }
 
       wxArrayString scList;
-      scList.Add(wxT("empname000048"));
-      scList.Add(wxT("empname000194"));
+      scList.Add(wxS("empname000048"));
+      scList.Add(wxS("empname000194"));
       sc.Bind(scList);
 
       wxSQLite3ResultSet q2 = db.ExecuteQuery("select empno from emp where empname in sc;");
@@ -450,7 +539,7 @@ int Minimal::OnRun()
       bool notNull;
       bool primaryKey;
       bool autoIncrement;
-      db.GetMetaData(wxEmptyString, wxT("emp"), wxT("empname"), &dataType, &collation, &notNull, &primaryKey, &autoIncrement);
+      db.GetMetaData(wxEmptyString, wxS("emp"), wxS("empname"), &dataType, &collation, &notNull, &primaryKey, &autoIncrement);
       cout << "Meta data of table 'emp', column 'empname'" << endl 
            << "Data type: " << (const char*) dataType.mb_str(wxConvUTF8)
            << ", Collation: " << (const char*) collation.mb_str(wxConvUTF8)
@@ -526,7 +615,7 @@ int Minimal::OnRun()
       cout << endl;
     }
 
-    db.CreateFunction(wxT("myagg"), 1, myAggregate);
+    db.CreateFunction(wxS("myagg"), 1, myAggregate);
 
     cout << endl << "Select statement test" << endl;
     wxSQLite3ResultSet q2 = db.ExecuteQuery("select myagg(empname) from emp order by 1;");
@@ -537,7 +626,7 @@ int Minimal::OnRun()
     }
 
 #if wxUSE_REGEX
-    db.CreateFunction(wxT("regexp"), 2, myRegExpOp);
+    db.CreateFunction(wxS("regexp"), 2, myRegExpOp);
 
     cout << endl << "Regular expression test" << endl;
     wxSQLite3ResultSet q3 = db.ExecuteQuery("select empname from emp where empname regexp '^[A-Z].*$' order by 1;");
@@ -553,29 +642,31 @@ int Minimal::OnRun()
 
     cout << endl << "Binary data test" << endl;
     db.ExecuteUpdate("create table bindata(desc char(10), data blob);");
-        
-    unsigned char bin[256];
-    for (i = 0; i < sizeof bin; i++)
+
+
+    int binSize = 256;
+    unsigned char* binData = new unsigned char[binSize];
+    for (i = 0; i < binSize; i++)
     {
-      bin[i] = i;
+      binData[i] = i;
     }
     wxSQLite3Statement stmt = db.PrepareStatement("insert into bindata values ('testing', ?);");
-    stmt.Bind(1,bin,sizeof bin);
+    stmt.Bind(1, binData, binSize);
     stmt.ExecuteUpdate();
-    cout << "Stored binary Length: " << sizeof bin << endl;
+    cout << "Stored binary Length: " << binSize << endl;
 
     q = db.ExecuteQuery("select data from bindata where desc = 'testing';");
 
-    const unsigned char* pbin = bin;
+    const unsigned char* pbin = binData;
     if (q.NextRow())
     {
       int blobLen;
-      const wxString columnName(wxT("data"));
-      pbin = q.GetBlob(wxString(wxT("data")),blobLen);
+      const wxString columnName(wxS("data"));
+      pbin = q.GetBlob(wxString(wxS("data")),blobLen);
       cout << "Retrieved binary Length: " << blobLen << endl;
     }
 
-    for (i = 0; i < sizeof bin; i++)
+    for (i = 0; i < sizeof binData; i++)
     {
       if (pbin[i] != i)
       {
@@ -583,7 +674,28 @@ int Minimal::OnRun()
       }
     }
     q.Finalize();
+    delete[] binData;
 
+    if (wxSQLite3Database::HasPointerParamsSupport())
+    {
+      static int aX[] = { 53, 9, 17, 2231, 4, 99 };
+      cout << endl << "Pointer parameter test" << endl;
+      wxSQLite3Statement stmt = db.PrepareStatement("SELECT * FROM carray(?,5);");
+      stmt.Bind(1, aX, wxString(wxS("carray")));
+      wxSQLite3ResultSet set = stmt.ExecuteQuery();
+
+      int count = 0;
+      while (set.NextRow())
+      {
+        int element = set.GetInt(0);
+        cout << "carray[" << count << "] = " << element << endl;
+        count++;
+      }
+      set.Finalize();
+      cout << "Is count == 5? " << (count == 5) << endl;
+    }
+
+    // Activate authorizer
     db.SetAuthorizer(myAuthorizer);
 
     // Pre-compiled Statements Demo
@@ -613,12 +725,15 @@ int Minimal::OnRun()
     cout << db.ExecuteScalar("select count(*) from emp;") << " rows in emp table in ";
     cout << tmEnd-tmStart << " seconds" << endl;
 
+    // Deactivate authorizer
+    db.RemoveAuthorizer();
+
     cout << endl << "User Defined Collation Sequence Test" << endl;
-    db.SetCollation(wxT("reversed"), &myCollation);
-    db.ExecuteUpdate(wxT("create table testcol(textcol char(20) collate reversed);"));
-    db.ExecuteUpdate(wxT("insert into testcol values ('anton');"));
-    db.ExecuteUpdate(wxT("insert into testcol values ('berta');"));
-    db.ExecuteUpdate(wxT("insert into testcol values ('cesar');"));
+    db.SetCollation(wxS("reversed"), &myCollation);
+    db.ExecuteUpdate(wxS("create table testcol(textcol char(20) collate reversed);"));
+    db.ExecuteUpdate(wxS("insert into testcol values ('anton');"));
+    db.ExecuteUpdate(wxS("insert into testcol values ('berta');"));
+    db.ExecuteUpdate(wxS("insert into testcol values ('cesar');"));
 
     wxSQLite3ResultSet q4 = db.ExecuteQuery("select textcol from testcol order by 1 desc;");
 
@@ -634,20 +749,20 @@ int Minimal::OnRun()
       cout << endl << "Backup and restore database" << endl;
       if (wxSQLite3Database::HasEncryptionSupport())
       {
-        db.Backup(&myProgress, wxGetCwd() + wxT("/test-backup.db"), wxT("password"));
+        db.Backup(&myProgress, wxGetCwd() + wxS("/test-backup.db"), wxS("password"));
       }
       else
       {
-        db.Backup(&myProgress, wxGetCwd() + wxT("/test-backup.db"));
+        db.Backup(&myProgress, wxGetCwd() + wxS("/test-backup.db"));
       }
       cout << endl << "... backup completed." << endl;
       if (wxSQLite3Database::HasEncryptionSupport())
       {
-        db.Restore(wxGetCwd() + wxT("/test-backup.db"), wxT("password"));
+        db.Restore(wxGetCwd() + wxS("/test-backup.db"), wxS("password"));
       }
       else
       {
-        db.Restore(wxGetCwd() + wxT("/test-backup.db"));
+        db.Restore(wxGetCwd() + wxS("/test-backup.db"));
       }
       cout << endl << "... restore completed." << endl;
     }
@@ -661,9 +776,9 @@ int Minimal::OnRun()
     if (wxSQLite3Database::HasIncrementalBlobSupport())
     {
       cout << endl << "Incremental BLOB test" << endl;
-      db.ExecuteUpdate(wxT("create table testblob(blobkey int, blobdata blob);"));
-      db.ExecuteUpdate(wxT("insert into testblob values (1, zeroblob(100));"));
-      wxSQLite3Blob incBlob = db.GetWritableBlob(1, wxT("blobdata"), wxT("testblob"));
+      db.ExecuteUpdate(wxS("create table testblob(blobkey int, blobdata blob);"));
+      db.ExecuteUpdate(wxS("insert into testblob values (1, zeroblob(100));"));
+      wxSQLite3Blob incBlob = db.GetWritableBlob(1, wxS("blobdata"), wxS("testblob"));
       int blobSize = incBlob.GetSize();
       int offset = 12;
       wxMemoryBuffer memBuffer;
@@ -671,7 +786,7 @@ int Minimal::OnRun()
       memBuffer.AppendData(myData, strlen(myData)+1);
       incBlob.Write(memBuffer, 12);
       incBlob.Finalize();
-      incBlob = db.GetReadOnlyBlob(1, wxT("blobdata"), wxT("testblob"));
+      incBlob = db.GetReadOnlyBlob(1, wxS("blobdata"), wxS("testblob"));
       incBlob.Read(memBuffer, (int) strlen(myData)+1, 12);
       incBlob.Finalize();
       char* readBlobData = (char*) memBuffer.GetData();
@@ -693,12 +808,16 @@ int Minimal::OnRun()
     cout << endl << "Test of RAII transactions" << endl;
     testTransaction();
 
+    // Test accessing encrypted database files (currently SQLCipher only)
+    TestCiphers();
+
     cout << endl << "End of tests" << endl;
     db.Close();
   }
   catch (wxSQLite3Exception& e)
   {
     cerr << e.GetErrorCode() << ":" << (const char*)(e.GetMessage().mb_str()) << endl;
+    m_rc = e.GetErrorCode();
   }
   
   try
@@ -709,17 +828,142 @@ int Minimal::OnRun()
   catch (wxSQLite3Exception& e)
   {
     cerr << e.GetErrorCode() << ":" << (const char*)(e.GetMessage().mb_str()) << endl;
+    m_rc = e.GetErrorCode();
   }
 
-  // Loop until user enters q or Q
-
-  char c(' ');
-  while (c != 'q' && c != 'Q')
+  if (!m_testMode)
   {
-    cout << "Press q then enter to quit: ";
-    cin >> c;
+    // Loop until user enters q or Q
+
+    char c(' ');
+    while (c != 'q' && c != 'Q')
+    {
+      cout << "Press q then enter to quit: ";
+      cin >> c;
+    }
   }
-  return 0;
+  return m_rc;
 }
 
+void Minimal::TestSQLCipher(wxSQLite3Cipher& cipher, const wxString& dbFileName, const wxString& dbKey)
+{
+  try
+  {
+    wxSQLite3Database db;
+    cout << endl << "Open database '" << (const char*) (dbFileName.mb_str()) << "'" << endl;
+    db.Open(dbFileName, cipher, dbKey);
+    wxString keySalt = db.GetKeySalt();
+    cout << "Cipher salt: " << (const char*) (keySalt.mb_str()) << endl;
+    int numRows = db.ExecuteScalar("SELECT COUNT(*) FROM t1");
+    cout << "Total number of rows = " << numRows << endl;
+    cout << "Distinct tuples:" << endl;
+    wxSQLite3ResultSet set = db.ExecuteQuery("SELECT DISTINCT * FROM t1");
+    int count = 0;
+    while (set.NextRow())
+    {
+      ++count;
+      wxString col1 = set.GetString(0);
+      wxString col2 = set.GetString(1);
+      cout << count << ": " << (const char*) (col1.mb_str()) << ", " << (const char*) (col2.mb_str()) << endl;
+    }
+    db.Close();
+  }
+  catch (wxSQLite3Exception& e)
+  {
+    cerr << e.GetErrorCode() << ":" << (const char*) (e.GetMessage().mb_str()) << endl;
+  }
+}
+
+void Minimal::TestPersonsDb(wxSQLite3Cipher& cipher, const wxString& dbFileName, const wxString& dbKey)
+{
+  try
+  {
+    wxSQLite3Database db;
+    cout << endl << "Open database '" << (const char*)(dbFileName.mb_str()) << "'" << endl;
+    db.Open(dbFileName, cipher, dbKey);
+    wxString keySalt = db.GetKeySalt();
+    cout << "Cipher salt: " << (const char*)(keySalt.mb_str()) << endl;
+    int numRows = db.ExecuteScalar("SELECT COUNT(*) FROM persons");
+    cout << "Total number of rows = " << numRows << endl;
+    cout << "Distinct persons in Rom:" << endl;
+    wxSQLite3ResultSet set = db.ExecuteQuery("SELECT DISTINCT lastname, firstname FROM persons WHERE city='Rom'");
+    int count = 0;
+    while (set.NextRow())
+    {
+      ++count;
+      wxString col1 = set.GetString(0);
+      wxString col2 = set.GetString(1);
+      cout << count << ": " << (const char*)(col1.mb_str()) << ", " << (const char*)(col2.mb_str()) << endl;
+    }
+    db.Close();
+  }
+  catch (wxSQLite3Exception& e)
+  {
+    cerr << e.GetErrorCode() << ":" << (const char*)(e.GetMessage().mb_str()) << endl;
+  }
+}
+
+void Minimal::TestCiphers()
+{
+  cout << endl << "Start testing ciphers" << endl;
+
+  // Test to access database encrypted with SQLCipher version 1
+  // Result: 75709 1 1 one one 1 2 one two 1 2
+  wxSQLite3CipherSQLCipher sqlCipher1;
+  sqlCipher1.InitializeVersionDefault(1);
+  TestSQLCipher(sqlCipher1, wxS("sqlcipher-1.1.8-testkey.db"), wxS("testkey"));
+
+  // Test to access database encrypted with SQLCipher version 2
+  // using 4000 iterations for the HMAC key derivation and a HMAC salt mask of zero
+  // Result: 38768 test-0-0 test-0-1 test-1-0 test-1-1
+  wxSQLite3CipherSQLCipher sqlCipher2;
+  sqlCipher2.InitializeVersionDefault(2);
+  sqlCipher2.SetFastKdfIter(4000);
+  sqlCipher2.SetHmacSaltMask(0);
+  TestSQLCipher(sqlCipher2, wxS("sqlcipher-2.0-beta-testkey.db"), wxS("testkey"));
+
+  // Test to access database encrypted with SQLCipher version 2
+  // using the page number in big endian form (BE) for the HMAC calculation
+  // Result: 78536 1 1 one one 1 2 one two
+  wxSQLite3CipherSQLCipher sqlCipher2be;
+  sqlCipher2be.InitializeVersionDefault(2);
+  sqlCipher2be.SetHmacPgNo(2);
+  TestSQLCipher(sqlCipher2be, wxS("sqlcipher-2.0-be-testkey.db"), wxS("testkey"));
+
+  // Test to access database encrypted with SQLCipher version 2
+  // using the page number in little endian form (LE) for the HMAC calculation
+  // Note: No change to the default initialization necessary
+  // Result: 78536 1 1 one one 1 2 one two
+  wxSQLite3CipherSQLCipher sqlCipher2le;
+  sqlCipher2le.InitializeVersionDefault(2);
+  TestSQLCipher(sqlCipher2le, wxS("sqlcipher-2.0-le-testkey.db"), wxS("testkey"));
+
+  // Test to access database encrypted with SQLCipher version 3
+  // Result: 78536 1 1 one one 1 2 one two
+  wxSQLite3CipherSQLCipher sqlCipher3;
+  sqlCipher3.InitializeVersionDefault(3);
+  TestSQLCipher(sqlCipher3, wxS("sqlcipher-3.0-testkey.db"), wxS("testkey"));
+
+  // Test to access database encrypted with SQLCipher version 4
+  // Result: 78536 1 1 one one 1 2 one two
+  wxSQLite3CipherSQLCipher sqlCipher4;
+  sqlCipher3.InitializeVersionDefault(4);
+  TestSQLCipher(sqlCipher3, wxS("sqlcipher-4.0-testkey.db"), wxS("testkey"));
+
+  // Test to access database encrypted with Ascon128
+  // Result: 200 (and 10 entries of names)
+  wxSQLite3CipherAscon128 ascon128;
+  ascon128.InitializeFromGlobalDefault();
+  TestPersonsDb(ascon128, wxS("persons-ascon128-testkey.db3"), wxS("testkey"));
+
+  // Test to access database encrypted with AEGIS
+  // Result: 200 (and 10 entries of names)
+  wxSQLite3CipherAegis aegis;
+  aegis.InitializeFromGlobalDefault();
+  TestPersonsDb(aegis, wxS("persons-aegis-testkey.db3"), wxS("testkey"));
+
+  cout << endl << "Finish testing ciphers" << endl;
+}
+
+DECLARE_APP(Minimal)
 IMPLEMENT_APP_CONSOLE(Minimal)
